@@ -160,6 +160,7 @@ namespace dbarts { namespace xval {
                               numInitialBurnIn, numContextShiftBurnIn, numRepBurnIn,
                               testSampleSize,
                               lossFunctorDef, numReps, cellParameters };
+    
     ext_rng_algorithm_t rng_algorithm = static_cast<ext_rng_algorithm_t>(threadControl.rng_algorithm);
     ext_rng_standardNormal_t rng_standardNormal = static_cast<ext_rng_standardNormal_t>(threadControl.rng_standardNormal);
     std::uint_least32_t rng_seed = threadControl.rng_seed;
@@ -204,7 +205,7 @@ namespace dbarts { namespace xval {
       
       ThreadData* threadData = misc_stackAllocate(numThreads, ThreadData);
       void** threadDataPtrs  = misc_stackAllocate(numThreads, void*);
-      for (size_t i = 0; i < offByOneIndex; ++i) {
+      for (size_t i = 0; i < numThreads; ++i) {
         threadData[i].shared = &sharedData;
         threadData[i].rng = createMultiThreadedRNG(
           rng_algorithm,
@@ -224,37 +225,16 @@ namespace dbarts { namespace xval {
           delete [] cellParameters;
           ext_throwError("unable to ensure seed uniqueness");
         }
-        threadData[i].repCellOffset = i * numRepCellsPerThread;
-        threadData[i].numRepCells   = numRepCellsPerThread;
         threadData[i].threadId = i;
-        threadData[i].results = results + i * numRepCellsPerThread * lossFunctorDef.numResults;
-        threadDataPtrs[i] = threadData + i;
-      }
-      
-      for (size_t i = offByOneIndex; i < numThreads; ++i) {
-        threadData[i].shared = &sharedData;
-        threadData[i].rng = createMultiThreadedRNG(
-          rng_algorithm,
-          rng_standardNormal,
-          seedGenerator,
-          errorMessage);
-        if (threadData[i].rng == NULL) {
-          for (size_t j = i; j > 0; --j) ext_rng_destroy(threadData[j - 1].rng);
-          if (seedGenerator != NULL) ext_rng_destroy(seedGenerator);
-          delete [] cellParameters;
-          ext_throwError(errorMessage);
+        if (i < offByOneIndex) {
+          threadData[i].numRepCells   = numRepCellsPerThread;
+          threadData[i].repCellOffset = i * numRepCellsPerThread;
+          threadData[i].results = results + i * numRepCellsPerThread * lossFunctorDef.numResults;
+        } else {
+          threadData[i].numRepCells   = numRepCellsPerThread - 1;
+          threadData[i].repCellOffset = offByOneIndex * numRepCellsPerThread + (i - offByOneIndex) * (numRepCellsPerThread - 1);
+          threadData[i].results = results + threadData[i].repCellOffset * lossFunctorDef.numResults;
         }
-        if (i > 0 && !ensureRNGSeedsAreUnique(threadData[i - 1].rng, threadData[i].rng, seedGenerator)) {
-          ext_rng_destroy(threadData[i].rng);
-          for (size_t j = i; j > 0; --j) ext_rng_destroy(threadData[j - 1].rng);
-          if (seedGenerator != NULL) ext_rng_destroy(seedGenerator);
-          delete [] cellParameters;
-          ext_throwError("unable to ensure seed uniqueness");
-        }
-        threadData[i].repCellOffset = offByOneIndex * numRepCellsPerThread + (i - offByOneIndex) * (numRepCellsPerThread - 1);
-        threadData[i].numRepCells   = numRepCellsPerThread - 1;
-        threadData[i].threadId = i;
-        threadData[i].results = results + threadData[i].repCellOffset * lossFunctorDef.numResults;
         threadDataPtrs[i] = threadData + i;
       }
       
@@ -289,6 +269,7 @@ namespace {
     size_t maxNumTestObservations;
     double* y_test;
     Results* samples;
+    double* weights;
     LossFunctor* lf;
     ext_rng* generator;
     size_t* permutation;
@@ -314,9 +295,11 @@ namespace {
                           misc_btm_manager_t manager, size_t threadId, bool lossRequiresMutex,
                           ThreadScratch* v_scratch);
   
-  void randomSubsampleDivideData(const Data& restrict origData, Data& restrict repData, double* restrict y_test,
+  void randomSubsampleDivideData(const Data& restrict origData, Data& restrict repData,
+                                 double* restrict y_test, double* restrict weights,
                                  ext_rng* restrict generator, size_t* restrict permutation);
-  void kFoldDivideData(const Data& restrict origData, Data& restrict repData, double* restrict y_test,
+  void kFoldDivideData(const Data& restrict origData, Data& restrict repData,
+                       double* restrict y_test, double* restrict weights,
                        size_t k, size_t maxNumFoldObservations, size_t numFullSizedFolds,
                        const size_t* restrict permutation);
 }
@@ -327,6 +310,7 @@ namespace {
     Method method;
     size_t maxNumTestObservations;
     size_t numSamples;
+    bool hasWeights;
     LossFunctor** result;
   };
   struct LossFunctorDestructorData {
@@ -337,7 +321,7 @@ namespace {
 
 extern "C" void lossFunctorCreatorTask(void* data) {
   LossFunctorCreatorData& lfcd(*static_cast<LossFunctorCreatorData*>(data));
-  *lfcd.result = lfcd.lfDef.createFunctor(lfcd.lfDef, lfcd.method, lfcd.maxNumTestObservations, lfcd.numSamples);
+  *lfcd.result = lfcd.lfDef.createFunctor(lfcd.lfDef, lfcd.method, lfcd.maxNumTestObservations, lfcd.numSamples, lfcd.hasWeights);
 }
 extern "C" void lossFunctorDestructorTask(void* data) {
   LossFunctorDestructorData& lfdd(*static_cast<LossFunctorDestructorData*>(data));
@@ -374,22 +358,32 @@ extern "C" {
         
     const LossFunctorDefinition& lfDef(sharedData.lossFunctorDef);
     bool lossRequiresMutex = lfDef.requiresMutex && !misc_btm_isNull(sharedData.threadManager);
+    bool hasWeights = origData.weights != NULL;
     
     LossFunctor* lf = NULL;
     if (lossRequiresMutex) {
-      LossFunctorCreatorData lfcd = { lfDef, sharedData.method, maxNumTestObservations, numSamples, &lf };
+      LossFunctorCreatorData lfcd = { lfDef, sharedData.method, maxNumTestObservations, numSamples, hasWeights, &lf };
       misc_btm_runTaskInParentThread(sharedData.threadManager, threadData.threadId, &lossFunctorCreatorTask, &lfcd);
     } else {
-      lf = lfDef.createFunctor(lfDef, sharedData.method, maxNumTestObservations, numSamples);
+      lf = lfDef.createFunctor(lfDef, sharedData.method, maxNumTestObservations, numSamples, hasWeights);
     }
     
     double* suppliedY_test      = lfDef.y_testOffset      >= 0 ?
                                   *reinterpret_cast<double**>(reinterpret_cast<char*>(lf) + lfDef.y_testOffset) :
                                   NULL;
-    double* suppliedTestSamples = sharedData.lossFunctorDef.testSamplesOffset >= 0 ?
+    double* suppliedTestSamples = lfDef.testSamplesOffset >= 0 ?
                                   *reinterpret_cast<double**>(reinterpret_cast<char*>(lf) + lfDef.testSamplesOffset) :
                                   NULL;
+    double* suppliedWeights = NULL;
+    if (hasWeights) {
+      suppliedWeights = lfDef.weightsOffset >= 0 ? 
+                        *reinterpret_cast<double**>(reinterpret_cast<char*>(lf) + lfDef.weightsOffset) :
+                        NULL;
+    }
     double* y_test = (suppliedY_test == NULL ? new double[maxNumTestObservations] : suppliedY_test);
+    double* weights = NULL;
+    if (hasWeights)
+      weights = (suppliedWeights == NULL ? new double[maxNumTestObservations] : suppliedWeights);
     
     Results* samples =
       suppliedTestSamples == NULL ?
@@ -412,7 +406,7 @@ extern "C" {
     allocateModelStorage(origModel, repModel);
     
     BARTFit* fit = new BARTFit(repControl, repModel, repData);
-    fit->state->rng = threadData.rng;
+    fit->state[0].rng = threadData.rng;
     
     CrossvalidationData xvalData = { *fit, origData, repData, 0 };
     
@@ -445,6 +439,7 @@ extern "C" {
     v_threadScratch->maxNumTrainingObservations = maxNumTrainingObservations;
     v_threadScratch->maxNumTestObservations = maxNumTestObservations;
     v_threadScratch->y_test = y_test;
+    v_threadScratch->weights = weights;
     v_threadScratch->samples = samples;
     v_threadScratch->lf = lf;
     v_threadScratch->generator = threadData.rng;
@@ -452,20 +447,36 @@ extern "C" {
     for (size_t i = 0; i < origData.numObservations; ++i) v_threadScratch->permutation[i] = i;
     
         
-    size_t firstCell    = threadData.repCellOffset / sharedData.numReps;
-    size_t firstCellRep = threadData.repCellOffset % sharedData.numReps;
-    size_t lastCell     = (threadData.repCellOffset + threadData.numRepCells) / sharedData.numReps;
-    size_t lastCellRep  = (threadData.repCellOffset + threadData.numRepCells) % sharedData.numReps;
-    
+    // each cell is handled completely by this thread
+    // cell-reps are partially handled by other threads; first is what we have to clean up from
+    // someone else, last is what we partially get into ourselves
+    //
+    // "Rows" are "cells", that is specific sets of hyperparameters
+    // "Cols" are the reps of each set of hyperpars
+    size_t firstCell = threadData.repCellOffset / sharedData.numReps; // start row
+    size_t firstCellRep = threadData.repCellOffset % sharedData.numReps; // start col
+    size_t lastCell = (threadData.repCellOffset + threadData.numRepCells - 1) / sharedData.numReps; // end row
+    size_t lastCellRep = (threadData.repCellOffset + threadData.numRepCells - 1) % sharedData.numReps + 1; // end col
+
     size_t resultIndex = 0;
     xvalData.numBurnIn = sharedData.numInitialBurnIn;
     
     // first and last cells are a bit of a mess, since there can be a lot of off-by-one stuff
+    // process the first incomplete row
     if (firstCellRep != 0) {
       updateFitForCell(*fit, repControl, repModel, sharedData.parameters[firstCell],
                        threadData.threadId, firstCell, sharedData.threadManager, verbose);
-      
-      for (size_t repIndex = firstCellRep; repIndex < sharedData.numReps; ++repIndex)
+      size_t repEnd;
+      if (firstCell == lastCell) {
+         // If all one row, handle it now and reset the end column
+         repEnd = lastCellRep;
+         lastCellRep = 0;
+      } else {
+         // Otherwise, handle next full row and whatever in next is necessary
+         repEnd = sharedData.numReps;
+         ++firstCell;
+      }
+      for (size_t repIndex = firstCellRep; repIndex < repEnd; ++repIndex)
       {
         crossvalidate(xvalData, samples, numSamples, threadData.results + resultIndex,
                       lfDef.calculateLoss, sharedData.threadManager, threadData.threadId, lossRequiresMutex, v_threadScratch);
@@ -475,9 +486,6 @@ extern "C" {
         xvalData.numBurnIn = sharedData.numRepBurnIn;
       }
       
-      ++firstCell;
-      firstCellRep = 0;
-      
       xvalData.numBurnIn = sharedData.numContextShiftBurnIn;
     }
     
@@ -521,6 +529,7 @@ extern "C" {
     }
     
     if (suppliedY_test == NULL) delete [] y_test;
+    if (hasWeights && suppliedWeights == NULL) delete [] weights;
     
     delete fit;
     
@@ -547,6 +556,7 @@ namespace {
     const double* y_test;
     size_t numTestObservations;
     const double* testSamples;
+    const double* weights;
     size_t numSamples;
     double* results;
   };
@@ -554,7 +564,7 @@ namespace {
 
 extern "C" void lossFunctorTask(void* data) {
   LossFunctorData& lfd(*static_cast<LossFunctorData*>(data));
-  lfd.calculateLoss(lfd.lf, lfd.y_test, lfd.numTestObservations, lfd.testSamples, lfd.numSamples, lfd.results);
+  lfd.calculateLoss(lfd.lf, lfd.y_test, lfd.numTestObservations, lfd.testSamples, lfd.numSamples, lfd.weights, lfd.results);
 }
 
 namespace {
@@ -565,17 +575,18 @@ namespace {
   {
     RandomSubsampleThreadScratch& threadScratch(*reinterpret_cast<RandomSubsampleThreadScratch *>(v_scratch));
     
-    randomSubsampleDivideData(xvalData.origData, xvalData.repData, threadScratch.y_test,
+    randomSubsampleDivideData(xvalData.origData, xvalData.repData, threadScratch.y_test, threadScratch.weights,
                               threadScratch.generator, threadScratch.permutation);
     xvalData.fit.setData(xvalData.repData);
 
     xvalData.fit.runSampler(xvalData.numBurnIn, samples);
     
     if (lossRequiresMutex) {
-      LossFunctorData ldf = { calculateLoss, *threadScratch.lf, threadScratch.y_test, threadScratch.maxNumTestObservations, samples->testSamples, numSamples, results };
+      LossFunctorData ldf = { calculateLoss, *threadScratch.lf, threadScratch.y_test, threadScratch.maxNumTestObservations, threadScratch.weights, samples->testSamples, numSamples, results };
       misc_btm_runTaskInParentThread(manager, threadId, &lossFunctorTask, &ldf);
     } else {
-      calculateLoss(*threadScratch.lf, threadScratch.y_test, threadScratch.maxNumTestObservations, samples->testSamples, numSamples, results);
+      calculateLoss(*threadScratch.lf, threadScratch.y_test, threadScratch.maxNumTestObservations,
+                    samples->testSamples, numSamples, threadScratch.weights, results);
     }
   }
   
@@ -612,17 +623,17 @@ namespace {
       xvalData.repData.numObservations = numTrainingObservations;
       xvalData.repData.numTestObservations = numTestObservations;
       
-      kFoldDivideData(xvalData.origData, xvalData.repData, threadScratch.y_test,
+      kFoldDivideData(xvalData.origData, xvalData.repData, threadScratch.y_test, threadScratch.weights,
                       k, threadScratch.maxNumTestObservations, threadScratch.numFullSizedFolds, threadScratch.permutation);
       xvalData.fit.setData(xvalData.repData);
       
       xvalData.fit.runSampler(xvalData.numBurnIn, samples);
     
       if (lossRequiresMutex) {
-        LossFunctorData ldf = { calculateLoss, *threadScratch.lf, threadScratch.y_test, numTestObservations, samples->testSamples, numSamples, foldResults };
+        LossFunctorData ldf = { calculateLoss, *threadScratch.lf, threadScratch.y_test, numTestObservations, samples->testSamples, threadScratch.weights, numSamples, foldResults };
         misc_btm_runTaskInParentThread(manager, threadId, &lossFunctorTask, &ldf);
       } else {
-        calculateLoss(*threadScratch.lf, threadScratch.y_test, numTestObservations, samples->testSamples, numSamples, foldResults);
+        calculateLoss(*threadScratch.lf, threadScratch.y_test, numTestObservations, samples->testSamples, numSamples, threadScratch.weights, foldResults);
       }
       
       for (size_t i = 0; i < threadScratch.numResults; ++i) results[i] += foldResults[i];
@@ -647,12 +658,14 @@ namespace {
     }
   }
   
-  void randomSubsampleDivideData(const Data& restrict origData, Data& restrict repData, double* restrict y_test,
+  void randomSubsampleDivideData(const Data& restrict origData, Data& restrict repData,
+                                 double* restrict y_test, double* restrict weights,
                                  ext_rng* restrict generator, size_t* restrict permutation)
   {
     double* restrict y = const_cast<double* restrict>(repData.y);
     double* restrict x = const_cast<double* restrict>(repData.x);
     double* restrict x_test = const_cast<double* restrict>(repData.x_test);
+    double* restrict repWeights = const_cast<double* restrict>(repData.weights);
     
     size_t numTrainingObservations = repData.numObservations;
     size_t numTestObservations     = repData.numTestObservations;
@@ -667,6 +680,7 @@ namespace {
       for (size_t j = 0; j < origData.numPredictors; ++j) {
         x_test[i + j * numTestObservations] = origData.x[obsIndex + j * origData.numObservations];
       }
+      if (weights != NULL) weights[i] = origData.weights[obsIndex];
     }
     for (size_t i = 0; i < numTrainingObservations; ++i) {
       size_t obsIndex = *permutation++;
@@ -674,6 +688,7 @@ namespace {
       for (size_t j = 0; j < origData.numPredictors; ++j) {
         x[i + j * numTrainingObservations] = origData.x[obsIndex + j * origData.numObservations];
       }
+      if (repWeights != NULL) repWeights[i] = origData.weights[obsIndex];
     }
     
     /* ext_printf("training data:\n");
@@ -693,7 +708,8 @@ namespace {
     ext_printf("\n"); */
   }
   
-  void kFoldDivideData(const Data& restrict origData, Data& restrict repData, double* restrict y_test,
+  void kFoldDivideData(const Data& restrict origData, Data& restrict repData,
+                       double* restrict y_test, double* restrict weights,
                        size_t k, size_t maxNumFoldObservations, size_t numFullSizedFolds,
                        const size_t* restrict permutation)
   {
@@ -701,6 +717,7 @@ namespace {
     double* restrict y = const_cast<double* restrict>(repData.y);
     double* restrict x = const_cast<double* restrict>(repData.x);
     double* restrict x_test = const_cast<double* restrict>(repData.x_test);
+    double* restrict repWeights = const_cast<double* restrict>(repData.weights);
     
     size_t numTrainingObservations = repData.numObservations;
     size_t numTestObservations     = repData.numTestObservations;
@@ -717,6 +734,8 @@ namespace {
       y_test[i] = origData.y[obsIndex];
       for (j = 0; j < origData.numPredictors; ++j)
         x_test[i + j * numTestObservations] = origData.x[obsIndex + j * origData.numObservations];
+      if (weights != NULL) weights[i] = origData.weights[obsIndex];
+
     }
     for (i = 0; i < foldStartIndex; ++i) {
       obsIndex = permutation[i];
@@ -724,6 +743,7 @@ namespace {
       for (j = 0; j < origData.numPredictors; ++j) {
         x[i + j * numTrainingObservations] = origData.x[obsIndex + j * origData.numObservations];
       }
+      if (repWeights != NULL) repWeights[i] = origData.weights[obsIndex];
     }
     for ( /* */; i < numTrainingObservations; ++i) {
       obsIndex = permutation[i + numTestObservations];
@@ -731,6 +751,7 @@ namespace {
       for (j = 0; j < origData.numPredictors; ++j) {
         x[i + j * numTrainingObservations] = origData.x[obsIndex + j * origData.numObservations];
       }
+      if (repWeights != NULL) repWeights[i] = origData.weights[obsIndex];
     }
   }
   
@@ -875,6 +896,7 @@ extern "C" void printTask(void* v_data) {
   if (data.k > 0.0) ext_printf("k: %.2f, ", data.k);
   ext_printf("power: %.2f, base: %.2f\n", data.power, data.base);
 }
+
 namespace {
   void updateFitForCell(BARTFit& fit, Control& repControl, Model& repModel, const CellParameters& parameters,
                         size_t threadId, size_t cellIndex, misc_btm_manager_t manager, bool verbose)
