@@ -173,7 +173,7 @@ int misc_htm_runTopLevelTasks(misc_htm_manager_t restrict manager, misc_htm_topL
   return result;
 }
 
-static void inline getTime(struct timespec* ts)
+static inline void getTime(struct timespec* ts)
 {
 #if defined(HAVE_CLOCK_GETTIME) && defined(CLOCK_REALTIME)
   clock_gettime(CLOCK_REALTIME, ts);
@@ -505,6 +505,8 @@ static int initializeManager(misc_htm_manager_t manager, size_t numThreads)
   
   manager->threadsShouldExit = false;
   
+  manager->buffer = NULL;
+  
   bool mutexInitialized = false;
   bool threadIsActiveInitialized = false;
   bool taskDoneInitialized = false;
